# SPDX-FileCopyrightText: 2023-2024 MTS (Mobile Telesystems)
# SPDX-License-Identifier: Apache-2.0
from horizon.backend.db.repositories.credentials_cache import CredentialsCacheRepository
from horizon.backend.db.repositories.hwm import HWMRepository
from horizon.backend.db.repositories.hwm_history import HWMHistoryRepository
from horizon.backend.db.repositories.namespace import NamespaceRepository
from horizon.backend.db.repositories.namespace_history import NamespaceHistoryRepository
from horizon.backend.db.repositories.user import UserRepository
