# SPDX-FileCopyrightText: 2023-2024 MTS (Mobile Telesystems)
# SPDX-License-Identifier: Apache-2.0

from horizon.backend.services.current_user import current_user
from horizon.backend.services.uow import UnitOfWork
